--[[
The Icarus Project, Copyright (c) 2005 John Forsythe <chaosmachine@gmail.com> (aka chaosmachine)
Feel free to learn and borrow from this code for non-commercial purposes.
If you use my code in your project, I'd appreciate a credit in your readme file.
]]

-- this menu uses object oriented lua :)
-- it was designed to be somewhat easy to reuse in other projects

Menu = {text="", pos = 0}

function Menu:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function Menu:setText(t)
	self.text = t
end

function Menu:getText()
	return self.text or ""
end

function Menu:getPos()
	return self.pos or 0
end

function Menu:left()
	-- do nothing by default
end
	
function Menu:right()
	-- do nothing by default
end

function Menu:push()
	-- do nothing by default
end
	
	

function showMenu()
	local menuItem = 0
	local exitMenu = false
	local m = {}
	-- menuItems = 6
	local jumpTo = level
	local pJumpTo = level
	local pSoundOn = soundOn
	local pMusicOn = musicOn
	
	updateScreen(1)
	updateScreen(1)
	
	-- set up menu objects
	
	menus = {}
	local i = 0
	
	-- toggle sound
	i=i+1; menus[i] = Menu:new{pos=1}
	
	aMenu = menus[i]
	function aMenu:getText()
		if soundOn then return "Sound: ON" else return "Sound: OFF" end					
	end
	
	function aMenu:left()
		soundOn = not soundOn
	end
	
	function aMenu:right()
		soundOn = not soundOn
	end
	
	function aMenu:push()
		soundOn = not soundOn
	end
	
	-- toggle music
	i=i+1; menus[i] = Menu:new{pos=2}
	
	aMenu = menus[i]
	function aMenu:getText()
		if musicOn then return "Music: ON" else return "Music: OFF" end					
	end
	
	function aMenu:left()
		toggleMusic()
	end
	
	function aMenu:right()
		toggleMusic()
	end
	
	function aMenu:push()
		toggleMusic()
	end
	
	-- jump to level
	i=i+1; menus[i] = Menu:new{pos=4}
	
	aMenu = menus[i]
	function aMenu:getText()
		return "Go to level: "..jumpTo
	end
	
	function aMenu:left()
		 -- change to max level beaten, not total levels
		if jumpTo > 1 then jumpTo = jumpTo - 1 else jumpTo = bestLevel end			
	end
		
	function aMenu:right()
		-- change to max level beaten, not total levels
		jumpTo = math.mod(jumpTo, bestLevel)+1
	end
	
	function aMenu:push()
		updateScreen(1)
		updateScreen(1)
		fadeOut()
		level = jumpTo
		startLevel(jumpTo)
		exitMenu = true
	end
		
	-- back to current level
	i=i+1; menus[i] = Menu:new{text="Return to game", pos=6}
	
	aMenu = menus[i]
	function aMenu:push()
		exitMenu = true
	end
	
	-- quit the game
	i=i+1; menus[i] = Menu:new{text = "Quit the game", pos=7}
	
	aMenu = menus[i]
	function aMenu:push()
		exitMenu = true
		exitNow = true
	end
	
	menuItems = table.getn(menus)
	
	-- menu loop
	
	while exitMenu == false do			
		-- draw menu layout
		menuHeight = 14+(10*9)
		menuWidth = 5+(8*16)
		menuTop = (272-menuHeight) / 2
		menuLeft = (480-menuWidth) / 2
		screen:fillRect(menuLeft-4,menuTop-4,menuWidth+8,menuHeight+8, colorBlack)
		screen:fillRect(menuLeft-1,menuTop-1,menuWidth+2,menuHeight+2, colorWhite)
		screen:fillRect(menuLeft,menuTop,menuWidth,menuHeight, colorBlack)
		screen:print(menuLeft+8,menuTop+8, "[Options]", colorWhite)
		-- show song in bottom right
		screen:fillRect(8,262,88,272, colorBlack)
		screen:print(8,262, "Song " .. song+1, colorWhite)
		
		-- draw menu items
		for i,aMenu in ipairs(menus) do
			if i == menuItem+1 then
				screen:print(menuLeft+8, menuTop+8+(10*(aMenu:getPos()+1)), aMenu:getText(), colorRed)
			else
				screen:print(menuLeft+8, menuTop+8+(10*(aMenu:getPos()+1)), aMenu:getText(), colorWhite)
			end
		end
		
		screen.waitVblankStart()
		screen.flip()
		
		-- check input
		local pad = Controls.read()
		
		if pad:select() then
			m.select = (m.select or 0)+1
		else
			if (m.select or 0) > 0 then -- button released
				exitMenu = true
			end
			m.select = 0
		end
		
		if pad:down() then
			m.down = (m.down or 0)+1
			if m.down == 1 or (m.down>18 and math.mod(m.down, 6)==0)then
				menuItem = math.mod(menuItem+1, menuItems)
			end
		else
			m.down = 0
		end
		
		if pad:up() then
			m.up = (m.up or 0)+1
			if m.up == 1 or (m.up>18 and math.mod(m.up, 6)==0) then
				if menuItem > 0 then menuItem = menuItem - 1 else menuItem = menuItems-1 end
			end
		else
			m.up = 0
		end	
				
		if pad:right() then
			m.right = (m.right or 0)+1
			if m.right == 1 or (m.right>18 and math.mod(m.right, 6)==0) then
				aMenu = menus[menuItem+1]
				aMenu:right()
			end
		else
			m.right = 0
		end

		if pad:left() then
			m.left = (m.left or 0)+1
			if m.left == 1 or (m.left>18 and math.mod(m.left, 6)==0) then
				menus[menuItem+1]:left()
			end
		else
			m.left = 0
		end
		
		if pad:square() or pad:triangle() or pad:circle() or pad:cross() or pad:start() then
			m.push = (m.push or 0)+1
			if m.push == 1 then
				menus[menuItem+1]:push()
			end
		else
			m.push = 0
		end
	end	
	
	-- save settings if something changed (there should be handling for keeping on the same level after restart)
	if not ((pSoundOn == soundOn) and (pMusicOn == musicOn)) then
		saveSettings()
	end
	
	doUpdateScreen = 1
end