--[[
The Icarus Project, Copyright (c) 2005 John Forsythe <chaosmachine@gmail.com> (aka chaosmachine)
Feel free to learn and borrow from this code for non-commercial purposes.
If you use my code in your project, I'd appreciate a credit in your readme file.
]]

function goToSleep()
	screen:fillRect(0, 0, 480, 272, colorBlack)
	screen:print(math.random(400),math.random(240), "sleeping",Color.new(255, 255, 255))
	screen.waitVblankStart()
	screen.flip()
	local doOnce = 0
	local pad = Controls.read()
	while pad == Controls.read() do	
		if math.mod(levelSeconds(),10) == 0 then
			if doOnce == 0 then
				doOnce = 1
				screen:fillRect(0, 0, 480, 272, colorBlack)
				screen:print(math.random(400),math.random(240), "sleeping",Color.new(255, 255, 255))
				screen.waitVblankStart()
				screen.flip()
			end
		else
			screen.waitVblankStart()
			doOnce = 0
		end
		
		if (not Music.playing()) and musicOn then
			nextSong()
		end
	end
	sleep=0
	updateScreen()
end

function levelSeconds()
	return os.difftime(os.time(), startTime)
end

function secondsToTime(seconds)
   local zh = math.floor(seconds / 3600)
   local zm = math.floor(seconds / 60) - zh * 60
   local zs = seconds - (zh * 3600 + zm * 60)
   if zh < 10 then zh = "0" .. zh end
   if zm < 10 then zm = "0" .. zm end
   if zs < 10 then zs = "0" .. zs end
   return zh .. ':' .. zm .. ':' .. zs
end

function startLevel(level, skipFade, keepTime)
	-- update settings if we're on a new level
	if level > bestLevel then
		bestLevel = level
	end
	lastLevel = level	
	saveSettings()
	
	background = Image.load("bkg/bkg"..level..".png")
	sourceTiles = Image.load("gfx/gfx"..level..".png")

	blocks = {}
	board = {}
	piece = {}
	border = {}
	p = {}
	goal = {}
	cursor = {x=15, y=8, lastX=15, lastY=8}
	
	oldPad = 0
	grabbed = 0
	sleep = 0
	lastCpsTime = 0
	dx = 0
	dy = 0
	doUpdateScreen = 0
	xOffset = 0
	yOffset = 0
	
	moveMode = false
	exitNow = false
		
	dofile("lvl/level" .. level .. ".lua")
	generateTiles()
	
	initPieceBlockTypes()
	initBorderBlockTypes()
	
	drawBorder()
	drawGoal()
	updateBlockImage()
	if not skipFade then fadeIn() end
	background:blit(0,0,fader)
	if not keepTime then startTime = os.time() end
	updateScreen()
end

function nextSong()
	if Music.playing() then Music.stop() end
	song = math.mod(song+1, songs)
	Music.playFile("mus/"..(song+1)..".xm")
	Music.volume(128)
end

function toggleMusic()
	musicOn = not musicOn
	if musicOn then
		nextSong()
	else
		if Music.playing() then Music.stop() end
	end
end

function wrongMove()
	if soundOn then
		badsound:play() badsound:play() badsound:play()
	end
end

function saveSettings()
	file = io.open("out/icarus.cfg", "w")

	local s = ""
	if soundOn then s=s.."1\n" else s=s.."0\n" end
	if musicOn then s=s.."1\n" else s=s.."0\n" end
	s=s..bestLevel.."\n"
	s=s..lastLevel
	
	file:write(s)
	file:close()
end


function loadSettings()
	file = io.open("out/icarus.cfg", "r")
	local c=0
	for line in file:lines() do
		c=c+1
		if c==1 then
			if line+0 == 1 then soundOn = true else soundOn = false end
		end
		if c==2 then
			if line+0 == 1 then musicOn = true else musicOn = false end
		end
		if c==3 then
			bestLevel = line+0
		end
		if c==4 then
			lastLevel = line+0
		end
	end
		
	file:close()
end

	-- debugging stuff below
	
-- function dumpPieces()
-- 	System.usbDiskModeDeactivate()
-- 	screen.waitVblankStart(180)
-- 	io.output(io.open(System.currentDirectory() .. "/out/piecedump.txt","w"))
-- 	
-- 	local pCount = 0
-- 	for i,aPiece in ipairs(piece) do
-- 		if aPiece.style ~= 4 then
-- 			pCount = pCount + 1
-- 			io.write("i=" .. pCount .. "\n")
-- 			io.write("piece[i] = {}\n\n")
-- 			for ii,aBlock in ipairs(aPiece) do
-- 				io.write("piece[i][" .. ii .. "] = {x=" .. aBlock.x .. ",y=" .. aBlock.y .. "}\n")
-- 			end
-- 			io.write("\n")
-- 			io.write("piece[i].x = " .. aPiece.x .. "\npiece[i].y = " .. aPiece.y .. "\n")
-- 			io.write("piece[i].style = " .. aPiece.style .. "\n\n")
-- 		end
-- 	end

-- 	io.close()
-- 	screen.waitVblankStart(180)
-- 	System.usbDiskModeActivate()
-- end
